import nsx_logging
from cli.utils.command_utils import invoke_command
from cli.utils.constants import NODE_TYPE_HOST_CENTOSKVM
from cli.utils.constants import NODE_TYPE_HOST_SLESKVM
from cli.utils.constants import NODE_TYPE_HOST_ESX
from cli.utils.constants import NODE_TYPE_HOST_RHELKVM
from cli.utils.constants import NODE_TYPE_HOST_UBUNTUKVM
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_CONTAINER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_CONTAINER
from cli.utils.node_utils import get_node_type
from cli.utils.error_code import ErrorCode as ec

lg = nsx_logging.getLogger("cli.commands.lcp_common.hyperbus.client")

_HYPERBUS_CLI = {
    NODE_TYPE_HOST_SLESKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/hyperbus/hyperbus-cli"
        },
    NODE_TYPE_HOST_CENTOSKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/hyperbus/hyperbus-cli"
        },
    NODE_TYPE_HOST_ESX: {
        'bin': "/opt/vmware/nsx-cli/bin/nsx-appctl",
        'socket': "/var/run/vmware/hyperbus/hyperbus-cli"
        },
    NODE_TYPE_HOST_RHELKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/hyperbus/hyperbus-cli"
        },
    NODE_TYPE_HOST_UBUNTUKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/hyperbus/hyperbus-cli"
        },
    NODE_TYPE_BAREMETAL_RHEL_CONTAINER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/hyperbus/hyperbus-cli"
        },
    NODE_TYPE_BAREMETAL_CENTOS_CONTAINER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/hyperbus/hyperbus-cli"
        },
    }


def has_hyperbus_cli(node_type):
    return node_type in (NODE_TYPE_BAREMETAL_CENTOS_CONTAINER,
                         NODE_TYPE_BAREMETAL_RHEL_CONTAINER,
                         NODE_TYPE_HOST_CENTOSKVM,
                         NODE_TYPE_HOST_SLESKVM,
                         NODE_TYPE_HOST_ESX,
                         NODE_TYPE_HOST_RHELKVM,
                         NODE_TYPE_HOST_UBUNTUKVM)


def invoke_hyperbus_appctl_command(cmd_list):
    lg.info("cmd_list: %s", cmd_list)
    node_type = get_node_type()
    if has_hyperbus_cli(node_type):
        cmd = [_HYPERBUS_CLI[node_type]['bin'], "-t", _HYPERBUS_CLI[node_type]['socket']]
        cmd.extend(cmd_list)
        lg.info("invoke_hyperbus_appctl_command: %s", cmd)
        rc, out, err = invoke_command(cmd)
        return (rc, out, err)
    else:
        lg.error("node type %s does not have hyperbus CLI", node_type,
                 error_code=ec.OPERATION_NOT_SUPPORTED)
        return (-1, None, "invalid command")


def get_hyperbus_cli_output(command, argv=[]):
    cmd_list = [command]
    if (len(argv)) > 0:
        cmd_list.extend(argv)
    rc, out, err = invoke_hyperbus_appctl_command(cmd_list)
    if rc != 0:
        lg.error("Failed to invoke hyperbus CLI command, rc: %d, err: %s", rc, err,
                 error_code=ec.OPERATION_FAILED)
        out = None

    return out
