# Copyright (C) 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

#logger.info("edge_dp data_store:{}".format(data_store))

profile = data_store['profile']
dp_stats = data_store['metrics']['nsx.edge.edge-dp-stats']

def encode_stats_msg(stats, msg):
    """ DP Stats """

    cpu_stats = stats.get('cpu_stats', None)
    mem_stats = stats.get('mem_pool_stats', None)
    kni_stats = stats.get('kni_intf_stats', None)
    phy_stats = stats.get('phy_intf_stats', None)

    if cpu_stats:
        for key, val in cpu_stats.items():
            try:
                busy_cycles = val['busy_cycles']
                idle_cycles = val['idle_cycles']
                percent_busy_cycles = int((busy_cycles/(busy_cycles + idle_cycles)) * 100)

                cpu_msg = nsx_edge_metrics_stats_pb2.NsxEdgeDpCpuCoreStats(obj_id = "core" + str(key),
                                                 packets_processed = val['packets_processed'],
                                                 percentage_busy_cycles = percent_busy_cycles)
            except KeyError:
                logger.info("EDGE_DP_STATS: keyerror, not sending cpu_stats for = {}".format(key))
                continue
            sub_msg = msg.dp_cpu_core.add()
            sub_msg.CopyFrom(cpu_msg)
    else:
        logger.debug("EDGE_DP_STATS: no cpu_stats to export")

    if mem_stats:
        for key, val in mem_stats.items():
            try:
                available_entries =  val['available_entries']
                size = val['size']
                if (size != 0):
                    percent_used_entries = int(((size - available_entries) * 100)/size)
                else:
                    percent_used_entries = 0

                mem_msg = nsx_edge_metrics_stats_pb2.NsxEdgeDpMemPoolStats(obj_id = key,
                                                 percentage_used_entries = percent_used_entries)

            except KeyError:
                logger.info("EDGE_DP_STATS: keyerror, not sending mem_stats for = {}".format(key))
                continue
            sub_msg = msg.dp_mem_pool.add()
            sub_msg.CopyFrom(mem_msg)
    else:
        logger.debug("EDGE_DP_STATS: no mem_stats to export")

    if kni_stats:
        for key, val in kni_stats.items():
            try:
                kni_msg = nsx_edge_metrics_stats_pb2.NsxEdgeDpKniIntfStats(obj_id = key,
                                                 rx_pkts  = val['stats_rx_pkts'],
                                                 rx_bytes = val['stats_rx_bytes'],
                                                 rx_drops = val['stats_rx_drops'],
                                                 tx_pkts  = val['stats_tx_pkts'],
                                                 tx_bytes = val['stats_tx_bytes'],
                                                 tx_drops = val['stats_tx_drops'])
            except KeyError:
                logger.info("EDGE_DP_STATS: keyerror, not sending kni_stats for = {}".format(key))
                continue
            sub_msg = msg.dp_kni_intf.add()
            sub_msg.CopyFrom(kni_msg)
    else:
        logger.debug("EDGE_DP_STATS: no kni_stats to export")

    if phy_stats:
        for key, val in phy_stats.items():
            try:
                phy_msg = nsx_edge_metrics_stats_pb2.NsxEdgeDpPhyIntfStats(obj_id = key,
                                                 rx_pkts           = val['rx_packets'],
                                                 rx_bytes          = val['rx_bytes'],
                                                 #This is not a port counter in itself. This will be displayed
                                                 #only if there is a switch that uses the port.
                                                 rx_drops_no_match = val.get('rx_drop_no_match',0),
                                                 rx_errors         = val['rx_errors'],
                                                 rx_misses         = val['rx_misses'],
                                                 rx_nombuf         = val['rx_nombufs'],
                                                 tx_packets        = val['tx_packets'],
                                                 tx_bytes          = val['tx_bytes'],
                                                 tx_errors         = val['tx_errors'],
                                                 tx_drops          = val['tx_drops'])
            except KeyError:
                logger.info("EDGE_DP_STATS: keyerror, not sending phy_stats for = {}".format(key))
                continue
            sub_msg = msg.dp_phy_intf.add()
            sub_msg.CopyFrom(phy_msg)
    else:
        logger.debug("EDGE_DP_STATS: no phy_stats to export")

    logger.debug("EDGE_DP_STATS: Encoded message {} is exported to SHA".format(msg))

def phy_inf_throughput_report(stats):
    throughput_stats = stats.get("phy_inf_throughput_stats", None)

    if throughput_stats:
        wavefront_source = throughput_stats[0]
        throughput_data = throughput_stats[1]

        logger.debug("Edge nic throughput: %s", throughput_data)

        for tput in throughput_data.get("throughput_data", []):
            Exportdata(data = tput,
                       exports = ['wavefront'],
                       source = wavefront_source,
                       timestamp = dp_stats['timestamp'])


def handle_stats(stats):
    entity_type = options_pb2.MetricsEntityType.Value('POLICY_EDGE_NODE')

    metrics_agent_msg = metrics_agent_rpc_pb2.MetricsAgentMessage(
                       timestamp = int(dp_stats['timestamp']),
                       entity = entity_type,
                       entity_id = str(data_store['host_id']))

    encode_stats_msg(stats, metrics_agent_msg)
    Exportdata(data = metrics_agent_msg, exports = ['tsdb_metrics'])

    phy_inf_throughput_report(stats)

if dp_stats and dp_stats['value']:
    handle_stats(dp_stats['value'])
else:
    logger.info("EDGE_DP_STATS: No DP stats for exporting.")
